# Code: Public Domain
require "rubygame"

module Rubygame

  attr_reader :queue

  Rubygame.init
  TTF.setup
  $font = TTF.new "FreeSans.ttf", 12 if File.exist? "FreeSans.ttf"
  $queue = EventQueue.new()
  $clock = Clock.new()
  $registered_events = []
  $tick_events = []

  # Register an event handler.
  #
  # Specify the conditions as parameter and give a block to execute when all the
  # conditions become true. The first parameter to the block is the event
  #
  # The conditions are either a list of symbols (e.g. ":key, :button") or a hash
  # whose value give some futher restriction (e.g. ":key => K_ESCAPE")
  #
  # =Conditions
  # :key:: A key was pressed. The value is either the key, a regexp (which means,
  #        the key's ASCI is matched against) or "true" (default) which matches any key
  # :tick:: True on every tick. Can only be specified as single argument (as
  #         no real other "events" occour during a tick)
  # :button:: The Mouse button of the mouse event. "true" for any mouse button
  # :mouse_motion:: The mouse has been moved.
  # :class:: An Array (or a single value) of any class that the event must be.
  #
  # =Examples
  # register :key => K_ESCAPE do throw :quit end  # ends game on escape
  # register :tick do world.update end            # call world.update on each tick
  # register :key => /awsd/ do |e| player.move e.key end  # call move for a,w,s or d
  # register 
  #
  def register *event, &block
    # Converting the event parameter to hash "c" (short for "conditions")
    c = {}
    event.each do |e|
      if e.is_a? Hash
        c.merge! e
      else
        c[e] = true
      end
    end
    c[:class] = [c[:class]] if c[:class] and not c[:class].is_a? Array
    # setting some sane defaults to :class
    if c[:key] and (c[:class].nil? or c[:class].select{|x|x.is_a?(KeyDownEvent)||x.is_a?(KeyUpEvent)}.empty?)
      (c[:class] ||= []) << KeyDownEvent
    end
    if c[:button] and (c[:class].nil? or c[:class].select{|x|x.is_a?(MouseDownEvent)||x.is_a?(MouseUpEvent)}.empty?)
      (c[:class] ||= []) << MouseDownEvent
    end
    $tick_events << block and return if c == {:tick => true}
    $registered_events << [c, block]
  end

  def start
    catch :quit do
      loop do
        $clock.tick()
        $tick_events.each{|e| e[nil]}
        $queue.each do |event|
          throw :quit if event.is_a? QuitEvent
          $registered_events.each do |c, block|
            block[event] if condition(c,event)
          end
          yield event if block_given?
        end
        $backbuffer.blit($screen, [0,0]) if $backbuffer and $screen
        $screen.update() if $screen
      end
    end
    Rubygame.quit
  end


  module Matrix2d
    def ysize
      size
    end
    
    def xsize
      at(0).size
    end

    def [] x,y = nil
      y.nil? ? at(x) : at(y)[x]
    end
    
    def []= x,y,v
      at(y)[x] = v
    end
    
    def each_index
      each_with_index do |line, y|
        line.each_with_index do |item, x|
          yield item,x,y
        end
      end
    end
  end
  
  class Surface
    def draw_string str, options = {}
      options[:font] ||= $font
      options[:color] ||= [255,255,255]
      options[:x] ||= 0
      options[:y] ||= 0

      options[:font].render(str.to_s, true, options[:color]).blit self, [options[:x], options[:y]]
    end
  end
  
end

# calculate whether the condition for an event was met
def condition c, e
  if c[:class]
    return false if c[:class].select{|x|e.is_a? x}.empty?
  end

  if c[:key]
    return false unless e.respond_to? :key
    if c[:key] != true
      if c[:key].is_a? Regexp
        return false if Key::KEY2ASCII[e.key] !~ c[:key]
      else
        return false if c[:key] != e.key
      end
    end
  end

  if c[:button]
    return false unless e.respond_to? :button
    return false if c[:button] != true and c[:button] != e.button
  end

  if c[:mouse_motion]
    return false unless e.is_a? MouseMotionEvent
  end

  true
end
